#include "tmesh.h"
#include <float.h>

void TMesh::SetCube(V3 center, float sl) {

  vertsN = 8;
  verts = new V3[vertsN];
  trisN = 12;
  tris = new unsigned int[trisN*3];

  verts[0] = center + V3(-sl/2.0f, +sl/2.0f, +sl/2.0f);
  verts[1] = center + V3(-sl/2.0f, -sl/2.0f, +sl/2.0f);
  verts[2] = center + V3(+sl/2.0f, -sl/2.0f, +sl/2.0f);
  verts[3] = center + V3(+sl/2.0f, +sl/2.0f, +sl/2.0f);
  verts[4] = center + V3(-sl/2.0f, +sl/2.0f, -sl/2.0f);
  verts[5] = center + V3(-sl/2.0f, -sl/2.0f, -sl/2.0f);
  verts[6] = center + V3(+sl/2.0f, -sl/2.0f, -sl/2.0f);
  verts[7] = center + V3(+sl/2.0f, +sl/2.0f, -sl/2.0f);

  int tri = 0;
  tris[3*tri+0] = 0;
  tris[3*tri+1] = 1;
  tris[3*tri+2] = 2;
  tri++;
  tris[3*tri+0] = 2;
  tris[3*tri+1] = 3;
  tris[3*tri+2] = 0;
  tri++;
  tris[3*tri+0] = 3;
  tris[3*tri+1] = 2;
  tris[3*tri+2] = 6;
  tri++;
  tris[3*tri+0] = 6;
  tris[3*tri+1] = 7;
  tris[3*tri+2] = 3;
  tri++;
  tris[3*tri+0] = 7;
  tris[3*tri+1] = 6;
  tris[3*tri+2] = 5;
  tri++;
  tris[3*tri+0] = 5;
  tris[3*tri+1] = 4;
  tris[3*tri+2] = 7;
  tri++;
  tris[3*tri+0] = 4;
  tris[3*tri+1] = 5;
  tris[3*tri+2] = 1;
  tri++;
  tris[3*tri+0] = 1;
  tris[3*tri+1] = 0;
  tris[3*tri+2] = 4;
  tri++;
  tris[3*tri+0] = 4;
  tris[3*tri+1] = 0;
  tris[3*tri+2] = 3;
  tri++;
  tris[3*tri+0] = 3;
  tris[3*tri+1] = 7;
  tris[3*tri+2] = 4;
  tri++;
  tris[3*tri+0] = 1;
  tris[3*tri+1] = 5;
  tris[3*tri+2] = 6;
  tri++;
  tris[3*tri+0] = 6;
  tris[3*tri+1] = 2;
  tris[3*tri+2] = 1;
  tri++;

  cols = new V3[vertsN];
  for (int i = 0; i < 4; i++) {
    cols[i] = V3(1.0f, 0.0f, 0.0f);
    cols[4+i] = V3(0.0f, 0.0f, 0.0f);
  }


}

void TMesh::Render(PPC *ppc, FrameBuffer *fb) {

  V3 *projVerts = new V3[vertsN];
  for (int vi = 0; vi < vertsN; vi++) {
    projVerts[vi] = V3(FLT_MAX, FLT_MAX, FLT_MAX);
    ppc->Project(verts[vi], projVerts[vi]);
  }

  for (int tri = 0; tri < trisN; tri++) {
    unsigned int vinds[3];
    vinds[0] = tris[3*tri+0];
    vinds[1] = tris[3*tri+1];
    vinds[2] = tris[3*tri+2];
    for (int ei = 0; ei < 3; ei++) {
      fb->DrawSegment(projVerts[vinds[ei]],
        projVerts[vinds[(ei+1)%3]],
        cols[vinds[ei]], cols[vinds[(ei+1)%3]]);
    }
  }

  delete []projVerts;

}

void TMesh::RotateAboutArbitraryAxis(V3 aO, 
  V3 aDir, float theta) {

  for (int vi = 0; vi < vertsN; vi++) {
    verts[vi] = verts[vi].RotatePointAboutArbitraryAxis(aO, aDir, theta);
  }


}

V3 TMesh::GetCenter() {

  V3 ret(0.0f, 0.0f, 0.0f);
  for (int vi = 0; vi < vertsN; vi++) {
    ret = ret + verts[vi];
  }
  return ret/(float)vertsN;

}